/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support 
 * ----------------------------------------------------------------------------
 * Copyright (c) 2008, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 */

//-----------------------------------------------------------------------------
//         Headers
//-----------------------------------------------------------------------------

#include <board.h>
#include <tc/tc.h>
#include <aic/aic.h>
#include <utility/trace.h>
#include <utility/assert.h>
#include "sys_arch.h"

#ifndef AT91C_ID_TC0
#if defined(AT91C_ID_TC012)
    #define AT91C_ID_TC0 AT91C_ID_TC012
#elif defined(AT91C_ID_TC)
    #define AT91C_ID_TC0 AT91C_ID_TC
#else
    #error Pb define ID_TC
#endif
#endif

//-----------------------------------------------------------------------------
///        Internal variables
//-----------------------------------------------------------------------------

/// clock tick count
static unsigned int clockTick;

//-----------------------------------------------------------------------------
///        Local functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Timer Counter 0 interrupt handler.
//-----------------------------------------------------------------------------
static void ISR_Tc0(void)
{
    // Acknowledge interrupt
    AT91C_BASE_TC0->TC_SR;

    // Increase tick
    clockTick ++;
}

//-----------------------------------------------------------------------------
///        Global functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/// Initialize for timing operation
//-----------------------------------------------------------------------------
void sys_init_timing(void)
{
    unsigned int div, tcclks;

    printf("P: clock time initialize - TC0\n\r");

    // Enable TC0
    AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_TC0;
    ASSERT(TC_FindMckDivisor(CLOCK_CONF_SECOND, BOARD_MCK, &div, &tcclks),
           "F: Fail to generate the desired frequency %d\n\r",
           CLOCK_CONF_SECOND);
    TC_Configure(AT91C_BASE_TC0,
                 tcclks | AT91C_TC_WAVE | AT91C_TC_WAVESEL_UP_AUTO);
    AT91C_BASE_TC0->TC_RC = BOARD_MCK / (CLOCK_CONF_SECOND * div);

    // Configure TC0 interrupt
    AIC_DisableIT(AT91C_ID_TC0);
    AIC_ConfigureIT(AT91C_ID_TC0, 0, ISR_Tc0);
    AT91C_BASE_TC0->TC_IER = AT91C_TC_CPCS;
    AIC_EnableIT(AT91C_ID_TC0);

    // Clear tick value
    clockTick = 0;

    // Start timer
    TC_Start(AT91C_BASE_TC0);
}

//-----------------------------------------------------------------------------
/// Read for clock time (ms)
//-----------------------------------------------------------------------------
unsigned int sys_get_ms(void)
{
    return clockTick;
}
/*---------------------------------------------------------------------------*/

